@@ -1,19 +1,27 @@
 "estlambda" <-
 function(data,plot=TRUE,proportion=1.0) {
-	if (min(data)<0) stop("data argument has values <0")
+	data <- data[which(!is.na(data))]
 	if (proportion>1.0 || proportion<=0) stop("proportion argument should be greater then zero and less than or equal to one")
-	ppoi <- ppoints(data)
 	ntp <- round(proportion*length(data))
-	if (ntp<10) stop("too few P-values provided")
+	if (ntp<1) stop("no valid measurments")
+	if (ntp==1) {
+		warning(paste("One measurment, Lambda = 1 returned"))
+		return(list(estimate=1.0,se=999.99))
+	}
+	if (ntp<10) warning(paste("number of points is too small:",ntp))
+	if (min(data)<0) stop("data argument has values <0")
 	if (max(data)<=1) {
-		data[data<1e-16] <- 1e-16
+		data[data==0] <- 1e-16
 		data <- qchisq(1-data,1)
 	}
 	data <- sort(data)
+	ppoi <- ppoints(data)
 	ppoi <- sort(qchisq(1-ppoi,1))
-	s <- summary(lm(data[1:ntp]~offset(ppoi[1:ntp])))$coeff
+	data <- data[1:ntp]
+	ppoi <- ppoi[1:ntp]
+	s <- summary(lm(data~offset(ppoi)))$coeff
 	if (plot) {
-		lim <- c(0,max(data,ppoi))
+		lim <- c(0,max(data,ppoi,na.rm=T))
 		plot(ppoi,data,xlim=lim,ylim=lim,xlab="Expected",ylab="Observed")
 		abline(a=0,b=1)
 		abline(a=0,b=(1+s[1,1]),col="red")
