@@ -18,6 +18,7 @@ function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,cl
 		if (length(tmeas) != data@nids) stop("Dimension of the outcome and SNP data object are different")
 		data <- data[tmeas,]
 		strata <- strata[tmeas]
+		resid <- resid[tmeas]
 	}
 	if (any(strata!=0)) {
 		olev <- levels(as.factor(strata))
@@ -35,11 +36,11 @@ function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,cl
 	out <- list()
 	for (j in c(1:(times+1*(times>1)))) {
 		if (j>1) resid <- sample(resid,replace=FALSE)
-		chi2 <- .C("grammar",as.raw(data@gtps),as.double(resid),as.double(h2object$InvSigma),as.integer(data@nids),as.integer(data@nsnps), as.integer(nstra), as.integer(strata), chi2 = double(6*data@nsnps), PACKAGE="GenABEL")$chi2
+		chi2 <- .C("grammar",as.raw(data@gtps),as.double(resid),as.double(h2object$InvSigma),as.integer(data@nids),as.integer(data@nsnps), as.integer(nstra), as.integer(strata), chi2 = double(7*data@nsnps), PACKAGE="GenABEL")$chi2
 		if (any(data@chromosome=="X")) {
 		  ogX <- data[,data@chromosome=="X"]
 		  sxstra <- strata; sxstra[ogX@male==1] <- strata[ogX@male==1]+nstra
-		  chi2X <- .C("grammar",as.raw(ogX@gtps),as.double(resid),as.double(h2object$InvSigma),as.integer(ogX@nids),as.integer(ogX@nsnps), as.integer(nstra*2), as.integer(sxstra), chi2 = double(6*ogX@nsnps), PACKAGE="GenABEL")$chi2
+		  chi2X <- .C("grammar",as.raw(ogX@gtps),as.double(resid),as.double(h2object$InvSigma),as.integer(ogX@nids),as.integer(ogX@nsnps), as.integer(nstra*2), as.integer(sxstra), chi2 = double(7*ogX@nsnps), PACKAGE="GenABEL")$chi2
 		  revec <- (data@chromosome=="X")
 		  revec <- rep(revec,6)
 		  chi2 <- replace(chi2,revec,chi2X)
@@ -112,6 +113,7 @@ function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,cl
 	out$idnames <- data@idnames
 	out$formula <- match.call()
 	out$family <- paste("score test for association with trait type") #,trait.type)
+	out$N <- chi2[(6*lenn+1):(lenn*7)]
 	class(out) <- "scan.gwaa"
 	out
 }
